"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
var argv = common_1.minimist(process.argv.slice(2));
var maxStringLength = function (strings) {
    return Math.max.apply(null, strings.map(function (item) { return item.length; }));
};
function printCommandHelp(name, command) {
    var args = Object.keys(command.args || {}).map(function (k) { return ({
        name: k,
        description: command.args ? command.args[k] : '',
    }); });
    var params = args.filter(function (item) { return !item.name.startsWith('-'); });
    var flags = args.filter(function (item) { return item.name.startsWith('-'); });
    var paramsDisplay = params.map(function (p) { return p.name; }).join(', ');
    var maxArgLength = maxStringLength(args.map(function (item) { return item.name; }));
    var logArg = function (arg, color) {
        var argName = ("" + arg.name + ' '.repeat(maxArgLength)).substr(0, maxArgLength);
        common_1.log.info("  " + common_1.log[color](argName) + "  " + arg.description);
    };
    common_1.log.info();
    common_1.log.info("Usage: " + common_1.log.blue(name) + " " + common_1.log.magenta(paramsDisplay));
    common_1.log.info.gray('------------------------------------------------------------');
    common_1.log.info("" + (command.description || 'No description.'));
    common_1.log.info();
    if (params.length > 0) {
        common_1.log.info.gray('Parameters:');
        params.forEach(function (arg) { return logArg(arg, 'magenta'); });
        common_1.log.info();
    }
    if (flags.length > 0) {
        common_1.log.info.gray('Flags:');
        flags.forEach(function (arg) { return logArg(arg, 'cyan'); });
        common_1.log.info();
    }
    common_1.log.info();
}
function toDisplayName(command) {
    var alias = command.alias, name = command.name;
    var displayName = name;
    if (alias.length > 0) {
        displayName = displayName + " (" + alias.join(',') + ")";
    }
    return displayName;
}
function printCommands(commands, maxNameLength) {
    var commandNames = Object.keys(commands).sort();
    commandNames.forEach(function (name) {
        var command = commands[name];
        var description = command.description;
        var displayName = toDisplayName(command);
        var paddedName = ("" + displayName + ' '.repeat(maxNameLength)).substr(0, maxNameLength);
        common_1.log.info("   " + common_1.log.blue(paddedName) + "  " + common_1.log.gray(description || 'No description.'));
    });
}
function printGroups(commands) {
    var displayNames = Object.keys(commands).map(function (k) {
        return toDisplayName(commands[k]);
    });
    var maxNameLength = maxStringLength(displayNames);
    var groups = common_1.toGroupedCommands(commands);
    Object.keys(groups).forEach(function (group) {
        if (group !== common_1.constants.DEFAULT_GROUP) {
            common_1.log.info();
            common_1.log.info.gray(" " + group);
            common_1.log.info();
        }
        printCommands(groups[group], maxNameLength);
    });
}
function findCommand(name, commands) {
    name = name || '';
    name = name.trim();
    if (commands[name]) {
        return commands[name];
    }
    return Object.keys(commands)
        .map(function (key) { return commands[key]; })
        .find(function (cmd) { return cmd.alias.includes(name); });
}
exports.default = (function (commands) {
    if (commands === void 0) { commands = {}; }
    var commandName = argv._[0];
    var command = findCommand(commandName, commands);
    var helpRequested = argv.h === true || argv.help === true;
    if (command === undefined) {
        common_1.log.info();
        common_1.log.info.cyan('Commands:\n');
        printGroups(commands);
        common_1.log.info();
        common_1.log.info();
        return;
    }
    else if (command && helpRequested) {
        printCommandHelp(commandName, command);
    }
    else {
        var args = argv;
        args._.shift();
        var commandArgs = {
            params: args._,
            options: args,
        };
        delete args._;
        if (command.validate && common_1.R.is(Function, command.validate)) {
            commandArgs = command.validate(commandArgs);
        }
        if (commandArgs && common_1.R.is(Function, command.action)) {
            try {
                var result = command.action(commandArgs);
                if (result && common_1.R.is(Function, result.catch)) {
                    result.catch(function (err) { return common_1.log.error(err, '\n'); });
                }
            }
            catch (err) {
                common_1.log.error(err, '\n');
            }
        }
    }
});
//# sourceMappingURL=command.js.map